"use strict";

exports.__esModule = true;
exports.default = Camera;

var _tab = require("./tools/tab");

var _dom = require("./tools/dom");

var _image = require("./tools/image");

var _position = require("./tools/position");

var _tools = require("./tools/tools");

function Camera(tabContainer, args) {
  var $ = (0, _tab.getQueryFunc)(tabContainer);
  var loading = (0, _tab.getLoadingFunc)(tabContainer);
  var pic = $('.pic');
  var video = $('.video');
  var events;

  function observe() {
    events = [(0, _dom.on)(video, 'loadedmetadata', videoPlay), (0, _dom.on)($('.take-photo'), 'click', takePhoto), (0, _dom.on)($('.take-another-one'), 'click', takeAnotherOne), (0, _dom.on)($('.use-it'), 'click', useIt), (0, _dom.on)(tabContainer, 'click', '.flip', function (e) {
      return flipIt(e.currTarget.getAttribute('data-dir'));
    })];
  }

  var flip, stream, streaming, blob;

  function _onEnter() {
    flip = {
      v: 1,
      h: 1
    };
    streaming = false;
    loading();

    _image.mediaDevices.getMedia({
      video: true
    }).catch(function () {
      loading(false);
      args.flash('camera_access_error', {
        back: true
      });
    }).then(function (s) {
      stream = s;

      if ('srcObject' in video) {
        video.srcObject = stream;
      } else {
        // Avoid using this in new browsers, as it is going away.
        video.src = URL.createObjectURL(stream);
      }

      video.play();
    });
  }

  function _onLeave() {
    if (stream) {
      if (stream.getTracks) (0, _tools.each)(stream.getTracks(), function (track) {
        track.stop();
      });
      if (blob) URL.revokeObjectURL(blob);

      try {
        URL.revokeObjectURL(stream);
      } catch (e) {}
    }

    takeAnotherOne();
  }

  function videoPlay() {
    if (!streaming) {
      streaming = true;
      loading(false);
      transformMedia();
    }
  }

  function takeAnotherOne() {
    pic.style.backgroundImage = '';
    (0, _dom.removeClass)(tabContainer, 'pic-mode');
  }

  function transformMedia() {
    (0, _position.transform)(video, 0, flip, 1);
    (0, _position.transform)(pic, 0, flip, 1);
  }

  function flipIt(prop) {
    flip[prop] *= -1;
    transformMedia();
  }

  function useIt() {
    args.onSelect(blob, 0, flip);
  }

  function takePhoto() {
    (0, _image.takePic)(video, function (b) {
      if (blob) URL.revokeObjectURL(blob);
      blob = b;
      pic.style.backgroundImage = "url('".concat(URL.createObjectURL(blob), "')");
    });
    (0, _dom.addClass)(tabContainer, 'pic-mode');
  }

  return {
    onEnter: function onEnter() {
      _onEnter();

      observe();
    },
    onLeave: function onLeave() {
      (0, _tab.stopObserving)(events);

      _onLeave();
    }
  };
}