"use strict";

exports.__esModule = true;
exports.default = Cropper;

var _tab = require("./tools/tab");

var _dom = require("./tools/dom");

var _image = require("./tools/image");

var _position = require("./tools/position");

var _tools = require("./tools/tools");

var ADJUST_RATIO = {
  left: function left(pos, otherSide, posLeft, posTop, posRight, posBottom, ratio) {
    // height: posBottom.bottom - posTop.top
    pos[otherSide] = Math.round(posRight.right - ratio.v * (posBottom.bottom - posTop.top) / ratio.h);
  },
  right: function right(pos, otherSide, posLeft, posTop, posRight, posBottom, ratio) {
    // height: posBottom.bottom - posTop.top
    pos[otherSide] = Math.round(posLeft.left + ratio.v * (posBottom.bottom - posTop.top) / ratio.h);
  },
  top: function top(pos, otherSide, posLeft, posTop, posRight, posBottom, ratio) {
    // width: posRight.right - posLeft.left
    pos[otherSide] = Math.round(posBottom.bottom - ratio.h * (posRight.right - posLeft.left) / ratio.v);
  },
  bottom: function bottom(pos, otherSide, posLeft, posTop, posRight, posBottom, ratio) {
    // width: posRight.right - posLeft.left
    pos[otherSide] = Math.round(posTop.top + ratio.h * (posRight.right - posLeft.left) / ratio.v);
  } // https://stackoverflow.com/questions/2752725/finding-whether-a-point-lies-inside-a-rectangle-or-not#37865332

};

function pointInRectangle(ax, ay, bx, by, cx, cy, px, py) {
  var abx = bx - ax;
  var aby = by - ay;
  var bcx = cx - bx;
  var bcy = cy - by;
  var dotABAP = abx * (px - ax) + aby * (py - ay);
  var dotBCBP = bcx * (px - bx) + bcy * (py - by);
  return 0 <= dotABAP && dotABAP <= abx * abx + aby * aby && 0 <= dotBCBP && dotBCBP <= bcx * bcx + bcy * bcy;
}

function Cropper(tabContainer, args) {
  var minWidth = args.minWidth;
  var minHeight = args.minHeight;
  var options = args.options.cropper || {};
  var flash = args.flash;
  var $ = (0, _tab.getQueryFunc)(tabContainer);
  var $$ = (0, _tab.getQueryAllFunc)(tabContainer);
  var loading = (0, _tab.getLoadingFunc)(tabContainer);
  var ratio = options.ratio;
  var viewportmeta = document.querySelector('meta[name="viewport"]');
  var mediaContainer = $('.media-container');
  var cropper = $('.cropper');
  var img = (0, _dom.hide)($('img', mediaContainer));
  var wheels = $$('.wheel');
  var preview = $('.preview img');

  preview.onload = function () {
    loading(false);
    URL.revokeObjectURL(uploadBlob);
  };

  preview.onerror = function () {
    loading(false);
    (0, _dom.removeClass)(tabContainer, 'preview-mode');
    flash('load_error');
  };

  img.onload = imageReady;

  img.onerror = function (e) {
    loading(false);
    flash('load_error', {
      back: true
    });
  };

  var imgStyle = img.style;
  var cropStyle = cropper.style;
  var blob, uploadBlob, rot, baseRot, scale, flip, imgWidth, imgHeight, handleInfo, imgInfo, mouseInfo, wheelInfo, zoomInfo, scaleStep, metaSave, params;
  var evCache = [];
  var prevDiff = -1;
  var events;

  function observe() {
    // TODO: touch support. Zoom...
    events = [(0, _dom.on)(tabContainer, 'touchstart,mousedown', '.handle,.cropper,img', pointerDown), (0, _dom.on)(tabContainer, 'touchstart,mousedown', '.wheel', wheelDown), // on(tabContainer, 'touchstart', '.img-container', zoomDown),
    (0, _dom.on)(document, 'touchend,touchcancel,mouseup', pointerUp), (0, _dom.on)(document, 'touchmove,mousemove', pointerMove), (0, _dom.on)(tabContainer, 'click', '.reset', reset), (0, _dom.on)(tabContainer, 'click', '.done', crop), (0, _dom.on)(tabContainer, 'click', '.rotate', function (e) {
      rotateIt(0, +e.currTarget.getAttribute('data-add'));
    }), (0, _dom.on)(tabContainer, 'click', '.flip', function (e) {
      flipIt(e.currTarget.getAttribute('data-dir'));
    }), (0, _dom.on)(tabContainer, 'click', '.zoom', function (e) {
      zoomIt(e.currTarget.getAttribute('data-zoom'));
    }), (0, _dom.on)(document, 'touchmove', function (e) {
      e.preventDefault();
    }, {
      passive: false
    }), (0, _dom.on)(tabContainer, 'click', '.back', function () {
      (0, _dom.removeClass)(tabContainer, 'preview-mode');
    }), (0, _dom.on)(tabContainer, 'click', '.upload', function () {
      args.upload(uploadBlob);
    }), (0, _dom.on)(document, 'contextmenu', function (e) {
      // Deactivate right click
      e.preventDefault();
    })];
  }

  function imagePos() {
    return {
      left: parseFloat(imgStyle.left),
      top: parseFloat(imgStyle.top)
    };
  }

  function cropperMetrics() {
    var left = parseFloat(cropStyle.left);
    var top = parseFloat(cropStyle.top);
    var width = parseFloat(cropStyle.width);
    var height = parseFloat(cropStyle.height);
    return {
      left: left,
      top: top,
      right: left + width,
      bottom: top + height,
      width: width,
      height: height
    };
  }

  function _onEnter(dataURI, r, f) {
    params = [r, f];
    loading();

    if (viewportmeta) {
      metaSave = viewportmeta.content;
      if (metaSave.indexOf('maximum-scale') < 0) viewportmeta.content += ',maximum-scale=1.0';
    }

    if ('string' !== typeof dataURI) {
      blob = dataURI;
      dataURI = URL.createObjectURL(dataURI);
    }

    init();
    img.src = dataURI;
  }

  function init() {
    baseRot = params[0] || 0;
    rot = 0;
    scale = 1;
    flip = params[1] || {
      h: 1,
      v: 1
    };
    imgStyle.top = 0;
    imgStyle.left = 0;
    cropper.style = '';
  }

  function reset() {
    init();
    imageReady();
  }

  function _onLeave() {
    (0, _dom.hide)(img);
    (0, _dom.removeClass)(tabContainer, 'preview-mode');
    if (metaSave) viewportmeta.content = metaSave;
  }

  function imageReady() {
    if (blob) {
      URL.revokeObjectURL(blob);
      blob = null;
    } // 2 next lines: fix issue with naturalWidth & naturalHeight inverted


    (0, _dom.show)(img);
    img.offsetHeight;
    var nw = img.naturalWidth;
    var nh = img.naturalHeight;

    if (nw < minWidth || nh < minHeight) {
      return flash('too_small', {
        back: true,
        values: {
          min: "".concat(minWidth, "x").concat(minHeight)
        }
      });
    }

    var invert = 90 === baseRot || 270 == baseRot;
    var dimImg = (0, _position.applyRatio)((invert ? mediaContainer.clientHeight : mediaContainer.clientWidth) - 20, (invert ? mediaContainer.clientWidth : mediaContainer.clientHeight) - 20, {
      v: nw,
      h: nh
    });

    if (dimImg.width > nw) {
      dimImg.width = nw;
      dimImg.height = nh;
    }

    (0, _position.transform)(img, baseRot + rot, flip, scale);
    imgStyle.width = dimImg.width + 'px';
    imgStyle.height = dimImg.height + 'px';
    imgWidth = dimImg.width;
    imgHeight = dimImg.height;
    scaleStep = Math.max((imgHeight + 80) / imgHeight, (imgWidth + 80) / imgWidth) - 1;

    if (ratio) {
      var dimCrop = (0, _position.applyRatio)(invert ? dimImg.height : dimImg.width, invert ? dimImg.width : dimImg.height, ratio);
      (0, _position.position)(cropStyle, {
        top: (dimImg.height - dimCrop.height) / 2,
        left: (dimImg.width - dimCrop.width) / 2,
        width: dimCrop.width,
        height: dimCrop.height
      });
    } else {
      (0, _position.position)(cropStyle, {
        top: 0,
        left: 0,
        width: dimImg.width,
        height: dimImg.height
      });
    }

    loading(false); // DEBUG
    // setInterval(() => {
    //   const imgPos = imagePos()
    //   const metrics = cropperMetrics()
    //   const inside = cropperInsideImg(
    //     scale,
    //     imgPos.left,
    //     imgPos.top,
    //     metrics.top,
    //     metrics.right,
    //     metrics.bottom,
    //     metrics.left,
    //     metrics.width,
    //     metrics.height,
    //   )
    //   cropper.classList[inside ? 'remove' : 'add']('outside')
    // }, 100)
  }

  function flipIt(prop) {
    flip[prop] *= -1;
    (0, _position.transform)(img, baseRot + rot, flip, scale);
  }

  function roundScale(n) {
    return 1 + Math.round((n - 1) / scaleStep) * scaleStep;
  }

  function zoomIt(prop) {
    var plus = prop === '+';
    scale = Math.max(0, roundScale(scale + (plus ? scaleStep : -scaleStep)));

    if (plus) {
      checkBigScale();
    } else {
      checkSmallScale();
    }

    (0, _position.transform)(img, baseRot + rot, flip, scale);
  }

  function rotateIt(add, addBase) {
    var oldRot = rot;
    rot = (rot + add + 360) % 360;
    if (rot > 180) rot -= 360;
    rot = Math.max(-45, Math.min(rot, 45));
    baseRot = (baseRot + (addBase || 0) + 360) % 360;
    checkSmallScale();
    var oldScale = scale;
    checkBigScale();

    if (oldScale !== scale) {
      scale = oldScale;
      return rot = oldRot;
    } // TODO: message (image size must be > ? move / resize cropper to rotate the image?...)


    (0, _position.transform)(img, baseRot + rot, flip, scale);
    (0, _tools.each)(wheels, function (wheel) {
      return (0, _position.transform)(wheel, rot, {
        h: 1,
        v: 1
      }, 1);
    });
  }

  function cropperInsideImg(imgScale, imgX, imgY, cropperLeft, cropperTop, cropperWidth, cropperHeight) {
    var widthRatio = img.naturalWidth / (imgWidth * imgScale);
    var heightRatio = img.naturalHeight / (imgHeight * imgScale);
    cropperLeft = Math.round((cropperLeft - imgX + imgWidth * (imgScale - 1) / 2) * widthRatio);
    cropperTop = Math.round((cropperTop - imgY + imgHeight * (imgScale - 1) / 2) * heightRatio);
    var cropperRight = Math.round(cropperLeft + cropperWidth * widthRatio);
    var cropperBottom = Math.round(cropperTop + cropperHeight * heightRatio);
    var rad = (baseRot + rot) * Math.PI / 180;
    var sin = Math.sin(-rad);
    var cos = Math.cos(-rad);
    var cw2 = img.naturalWidth / 2;
    var ch2 = img.naturalHeight / 2;
    var ypSin = cw2 * sin;
    var ypCos = ch2 * cos;
    var xpCos = cw2 * cos;
    var xpSin = ch2 * sin;
    var xp0 = Math.round(cw2 + -xpCos - xpSin);
    var xp1 = Math.round(cw2 + -xpCos + xpSin);
    var xp2 = Math.round(cw2 + xpCos + xpSin);
    var yp0 = Math.round(ch2 + ypSin - ypCos); // tl

    var yp1 = Math.round(ch2 + ypSin + ypCos); // bl

    var yp2 = Math.round(ch2 + -ypSin + ypCos); // br

    return pointInRectangle(xp0, yp0, xp1, yp1, xp2, yp2, cropperLeft, cropperTop) && pointInRectangle(xp0, yp0, xp1, yp1, xp2, yp2, cropperLeft, cropperBottom) && pointInRectangle(xp0, yp0, xp1, yp1, xp2, yp2, cropperRight, cropperBottom) && pointInRectangle(xp0, yp0, xp1, yp1, xp2, yp2, cropperRight, cropperTop);
  }

  function checkBigScale() {
    if (minWidth) {
      // minSize = (imgWidth * minWidth) / img.naturalWidth
      scale = Math.min(scale, cropperMetrics().width / (imgWidth * minWidth / img.naturalWidth)); // TODO: message ? Size must be min AxB ?
    }

    if (minHeight) {
      // minSize = (imgHeight * minHeight) / img.naturalHeight
      scale = Math.min(scale, cropperMetrics().height / (imgHeight * minHeight / img.naturalHeight));
    }
  }

  function checkSmallScale() {
    var imgPos = imagePos();
    var metrics = cropperMetrics();

    var inside = function inside(scale) {
      return cropperInsideImg(scale, imgPos.left, imgPos.top, metrics.left, metrics.top, metrics.width, metrics.height);
    };

    if (!inside(scale)) {
      var maxSize = Math.sqrt(Math.pow(img.naturalWidth * (1 + scale), 2) + Math.pow(img.naturalHeight * (1 + scale), 2)) + 1;
      var originalSize = Math.max(imgWidth, imgHeight);
      var size = Math.round(originalSize * scale);
      var count = Math.floor(Math.log10(Math.abs(maxSize - size)));
      var unit;

      while (count > 0) {
        unit = Math.pow(10, --count);

        while (!inside(size / originalSize)) {
          size += unit;
        }

        if (1 !== unit) size -= unit;
      }

      scale = size / originalSize;
    }
  }

  function crop() {
    loading();
    var widthRatio = img.naturalWidth / (imgWidth * scale);
    var heightRatio = img.naturalHeight / (imgHeight * scale);
    var cropMetrics = cropperMetrics();
    var imgPos = imagePos();
    var width = cropMetrics.width * widthRatio;
    var height = cropMetrics.height * heightRatio;
    var resizeToWidth = options.resizeToWidth;

    if (!options.upscale && width <= resizeToWidth) {
      resizeToWidth = null;
    }

    (0, _image.transformPic)(img, {
      left: (cropMetrics.left - imgPos.left + imgWidth * (scale - 1) / 2) * widthRatio,
      top: (cropMetrics.top - imgPos.top + imgHeight * (scale - 1) / 2) * heightRatio,
      width: width,
      height: height,
      rotate: baseRot + rot,
      flip: flip,
      scale: scale,
      resizeToWidth: resizeToWidth,
      resizeToHeight: ratio ? resizeToWidth * ratio.h / ratio.v : resizeToWidth * cropMetrics.height / cropMetrics.width,
      webp: options.webpIfSupported
    }, function (blob) {
      uploadBlob = blob;
      preview.src = URL.createObjectURL(blob);
      (0, _dom.addClass)(tabContainer, 'preview-mode');
    });
  } // function zoomDown(e) {
  //   // setTimeout(() => {
  //   //   alert(e.touches.length)
  //   // }, 1000)
  // }
  // function zoomMove(e) {}
  // function zoomUp(e) {}


  function getEvent(e) {
    return e.touches ? e.touches[0] : e;
  }

  function wheelDown(e) {
    var event = getEvent(e);
    wheelInfo = {
      scale: scale,
      dir: e.currTarget.getAttribute('data-dir')
    };
    mouseInfo = {
      x: event.clientX,
      y: event.clientY
    };
  }

  function wheelMove(diffX, diffY) {
    scale = wheelInfo.scale;
    var diff = 'h' === wheelInfo.dir ? -diffY : diffX;

    if (rot > 0 && rot <= 4 && diff < 0 || rot < 0 && rot >= -4 && diff > 0) {
      diff = -rot;
    } else {
      diff = Math.round(Math.round(diff) * 2) / 4;
    }

    rotateIt(diff);
  }

  function wheelUp() {
    wheelInfo = null;
  }

  function pointerDown(e) {
    e.preventDefault();
    var event = getEvent(e);

    if (e.currTarget.matches('.handle')) {
      (0, _dom.addClass)(tabContainer, 'cropping');
      var dataPos = e.currTarget.getAttribute('data-pos');
      handleInfo = {
        pos: cropperMetrics(),
        originalPos: cropperMetrics(),
        min: {
          width: minWidth ? Math.ceil(imgWidth * scale * minWidth / img.naturalWidth) : 0,
          height: minHeight ? Math.ceil(imgHeight * scale * minHeight / img.naturalHeight) : 0
        },
        handle: {
          left: dataPos.indexOf('l') >= 0,
          right: dataPos.indexOf('r') >= 0,
          top: dataPos.indexOf('t') >= 0,
          bottom: dataPos.indexOf('b') >= 0
        }
      };
    } else {
      imgInfo = {
        pos: imagePos(),
        originalPos: imagePos()
      };
    }

    mouseInfo = {
      x: event.clientX,
      y: event.clientY
    };
  }

  function handleUp() {
    (0, _dom.removeClass)(tabContainer, 'cropping');
    var pos = cropperMetrics();
    var imgPos = imagePos();
    var oPos = handleInfo.originalPos;
    var handle = handleInfo.handle;

    var inside = function inside(top, right, bottom, left) {
      return cropperInsideImg(scale, imgPos.left, imgPos.top, left, top, right - left, bottom - top);
    };

    if (!inside(pos.top, pos.right, pos.bottom, pos.left)) {
      var sides = [handle.left ? 'left' : 'right', handle.top ? 'top' : 'bottom'];
      var otherSide; // Only left or right side needs to be adjusted

      if (inside(pos.top, (handle.right ? oPos : pos).right, pos.bottom, (handle.left ? oPos : pos).left)) {
        otherSide = sides.pop();
        if (!ratio) oPos[otherSide] = pos[otherSide];
      } // Only top or bottom side needs to be adjusted


      if (inside((handle.top ? oPos : pos).top, pos.right, (handle.bottom ? oPos : pos).bottom, pos.left)) {
        otherSide = sides.shift();
        if (!ratio) oPos[otherSide] = pos[otherSide];
      } // If ratio needs to be applied, just ensure we calculate the width/height from the biggest diff


      if (ratio && sides.length > 1) {
        if (oPos[sides[0]] - pos[sides[0]] < oPos[sides[1]] - pos[sides[1]]) {
          sides.reverse();
        }

        otherSide = sides.pop();
      }

      var adjustRatio = ADJUST_RATIO[otherSide];
      var diff, count, original, lastPos, unit, width, height, within, posLeft, posTop, posRight, posBottom;

      for (var i = 0, len = sides.length, side; i < len; ++i) {
        side = sides[i];
        diff = oPos[side] - pos[side];
        count = Math.floor(Math.log10(Math.abs(diff)));
        original = pos[side];
        posLeft = 'left' === side || 'left' === otherSide ? pos : oPos;
        posTop = 'top' === side || 'top' === otherSide ? pos : oPos;
        posRight = 'right' === side || 'right' === otherSide ? pos : oPos;
        posBottom = 'bottom' === side || 'bottom' === otherSide ? pos : oPos;
        pos[side] = oPos[side];
        if (ratio) pos[otherSide] = oPos[otherSide];

        while (count > 0) {
          unit = Math.pow(10, --count) * (diff < 0 ? -1 : 1);

          while (inside(posTop.top, posRight.right, posBottom.bottom, posLeft.left) && (diff > 0 ? pos[side] >= original : pos[side] <= original)) {
            pos[side] -= unit;

            if (ratio) {
              lastPos = pos[otherSide];
              adjustRatio(pos, otherSide, posLeft, posTop, posRight, posBottom, ratio);
            }
          }

          pos[side] += unit;
          if (ratio) pos[otherSide] = lastPos;
        }

        oPos[side] = pos[side];
      }
    }

    pos.width = pos.right - pos.left;
    pos.height = pos.bottom - pos.top;
    (0, _position.position)(cropStyle, pos);
    handleInfo = null;
  }

  function imgUp() {
    var pos = imagePos();
    var oPos = imgInfo.originalPos;
    var cPos = cropperMetrics();

    var inside = function inside(imgX, imgY) {
      return cropperInsideImg(scale, imgX, imgY, cPos.left, cPos.top, cPos.width, cPos.height);
    };

    if (!inside(pos.left, pos.top)) {
      var sides = ['left', 'top'];
      var side; // Only left or top side needs to be adjusted

      if (inside(pos.left, oPos.top)) {
        side = sides.shift();
        oPos[side] = pos[side];
      } else if (inside(oPos.left, pos.top)) {
        side = sides.pop();
        oPos[side] = pos[side];
      }

      var diff, count, original, oPosX, oPosY, unit;

      for (var i = 0, len = sides.length, _side; i < len; ++i) {
        _side = sides[i];
        diff = oPos[_side] - pos[_side];
        count = Math.floor(Math.log10(Math.abs(diff)));
        original = pos[_side];
        oPosX = 'left' === _side ? pos : oPos;
        oPosY = 'top' === _side ? pos : oPos;
        pos[_side] = oPos[_side];

        while (count > 0) {
          unit = Math.pow(10, --count) * (diff < 0 ? -1 : 1);

          while (inside(oPosX.left, oPosY.top) && (diff > 0 ? pos[_side] >= original : pos[_side] <= original)) {
            pos[_side] -= unit;
          }

          pos[_side] += unit;
        }

        oPos[_side] = pos[_side];
      }
    }

    imgStyle.top = pos.top + 'px';
    imgStyle.left = pos.left + 'px';
    imgInfo = null;
  }

  function pointerUp(e) {
    if (handleInfo) handleUp();else if (imgInfo) imgUp();else if (wheelInfo) wheelUp();
  }

  function pointerMove(e) {
    if (handleInfo || imgInfo || wheelInfo) {
      var event = getEvent(e);
      var clientX = event.clientX;
      var clientY = event.clientY;
      (handleInfo ? handleMove : imgInfo ? imgMove : wheelMove)(Math.round(mouseInfo.x - clientX), Math.round(mouseInfo.y - clientY));
      mouseInfo = {
        x: clientX,
        y: clientY
      };
    }
  }

  function handleMove(diffX, diffY) {
    var pos = handleInfo.pos;
    var handle = handleInfo.handle;
    var min = handleInfo.min;
    if (handle.top) pos.top -= diffY;
    if (handle.left) pos.left -= diffX;
    if (handle.bottom) pos.bottom -= diffY;
    if (handle.right) pos.right -= diffX;
    var dim = ratio ? (0, _position.applyRatio)(pos.right - pos.left, pos.bottom - pos.top, ratio) : null;
    var left = handle.left ? Math.min(pos.right - min.width, ratio ? pos.right - dim.width : pos.left) : pos.left;
    var top = handle.top ? Math.min(pos.bottom - min.height, ratio ? pos.bottom - dim.height : pos.top) : pos.top;
    var right = handle.right ? Math.max(pos.left + min.width, ratio ? pos.left + dim.width : pos.right) : pos.right;
    var bottom = handle.bottom ? Math.max(pos.top + min.height, ratio ? pos.top + dim.height : pos.bottom) : pos.bottom;
    (0, _position.position)(cropStyle, {
      left: left,
      top: top,
      width: right - left,
      height: bottom - top
    });
  }

  function imgMove(diffX, diffY) {
    var pos = imgInfo.pos;
    imgStyle.top = (pos.top -= diffY) + 'px';
    imgStyle.left = (pos.left -= diffX) + 'px';
  }

  return {
    onEnter: function onEnter(dataURI, rot, flip) {
      _onEnter(dataURI, rot, flip);

      observe();
    },
    onLeave: function onLeave() {
      (0, _tab.stopObserving)(events);

      _onLeave();
    },
    onOrientationChange: reset
  };
}