"use strict";

exports.__esModule = true;
exports.default = Local;

var _tab = require("./tools/tab");

var _isNode = _interopRequireDefault(require("./tools/is-node"));

var _dom = require("./tools/dom");

var _image = require("./tools/image");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DRAG_N_DROP_SUPPORT = _isNode.default ? false : function (div) {
  return 'draggable' in div || 'ondragstart' in div && 'ondrop' in div;
}(document.createElement('div'));
var ACCEPT_SEP_REG = /\s*,\s*/;
var ESCAPE_REG = /([.+?^=!:${}()|[\]\/\\])/g; // Removed star char

var ANY_REG = /\*/g;

function Local(tabContainer, args) {
  var flash = args.flash;
  var options = args.options.local || {};
  options.accept = options.accept || 'image/jpeg,image/gif,image/png,image/webp';
  var $ = (0, _tab.getQueryFunc)(tabContainer);
  var loading = (0, _tab.getLoadingFunc)(tabContainer);
  var dropArea = $('.drop-area');
  var input = $('input');
  var accept = options.accept.trim().toLowerCase().split(ACCEPT_SEP_REG).reduce(function (acc, type) {
    if (type && (_image.webPSupport || 'image/webp' !== type)) {
      acc.push(type.charAt(0) === '.' || type.indexOf('*') < 0 ? type : new RegExp('^' + type.replace(ESCAPE_REG, '\\$1').replace(ANY_REG, '.*') + '$', 'i'));
    }

    return acc;
  }, []);
  if (accept) input.setAttribute('accept', options.accept);
  if (!DRAG_N_DROP_SUPPORT) (0, _dom.addClass)(dropArea, 'no-support');
  var events;

  function observe() {
    events = [(0, _dom.on)(input, 'change', function (e) {
      handleFile(input.files[0]);
      input.value = null;
    })];

    if (DRAG_N_DROP_SUPPORT) {
      events = events.concat(['drag', 'dragstart', 'dragend', 'dragenter', 'dragover', 'dragleave', 'drop'].map(function (eventName) {
        return (0, _dom.on)(dropArea, eventName, preventDefaults);
      }), ['dragenter', 'dragover'].map(function (eventName) {
        return (0, _dom.on)(dropArea, eventName, highlight);
      }), ['dragleave', 'dragend', 'drop'].map(function (eventName) {
        return (0, _dom.on)(dropArea, eventName, unhighlight);
      }), (0, _dom.on)(dropArea, 'drop', function (e) {
        return handleFile(e.dataTransfer.files[0]);
      }));
    }
  }

  function handleFile(file) {
    if (!file) return; // Validation

    var ftype = (file.type || '').toLowerCase();
    var fext = fileExt(file.name || '');

    if (accept && !accept.some(function (type) {
      return typeof type === 'string' ? type === (type.charAt(0) === '.' ? fext : ftype) : type.test(ftype);
    })) {
      return flash('invalid_file_type');
    }

    loading(); // Convert gif to stop animation

    if ('image/gif' === file.type) {
      return (0, _image.convertGif)(file, function (dataURI) {
        loading(false);
        args.onSelect(dataURI);
      }, function () {
        loading(false);
        flash('load_error');
      });
    }

    (0, _image.getOrientation)(file, function (orientation) {
      loading(false);
      args.onSelect(file, // Rotation
      (5 === orientation || 6 === orientation ? 1 : 3 === orientation || 4 === orientation ? 2 : 7 === orientation || 8 === orientation ? 3 : 0) * 90, // Flip
      {
        h: [2, 4, 5, 7].indexOf(orientation) >= 0 ? -1 : 1,
        v: 1
      });
    });
  }

  function highlight() {
    (0, _dom.addClass)(dropArea, 'highlight');
  }

  function unhighlight() {
    (0, _dom.removeClass)(dropArea, 'highlight');
  }

  return {
    onEnter: observe,
    onLeave: function onLeave() {
      return (0, _tab.stopObserving)(events);
    }
  };
}

function preventDefaults(e) {
  e.preventDefault();
  e.stopPropagation();
}

function fileExt(filename) {
  return filename.slice((filename.lastIndexOf('.') - 1 >>> 0) + 1).toLowerCase();
}