"use strict";

exports.__esModule = true;
exports.default = Source;
exports.DEFAULT_TIMEOUT = exports.DEFAULT_LOAD_ITEMS = void 0;

var _tab = require("./tools/tab");

var _dom = require("./tools/dom");

var _tools = require("./tools/tools");

var DEFAULT_LOAD_ITEMS = 24;
exports.DEFAULT_LOAD_ITEMS = DEFAULT_LOAD_ITEMS;
var DEFAULT_TIMEOUT = 20 * 60 * 1000;
exports.DEFAULT_TIMEOUT = DEFAULT_TIMEOUT;

function Source(tabContainer, args, onConnect, loadMorePhotos, loadPhotos, loadMoreAlbums) {
  var flash = args.flash;
  var i18n = args.i18n;
  var $ = (0, _tab.getQueryFunc)(tabContainer);
  var $$ = (0, _tab.getQueryAllFunc)(tabContainer);
  var loading = (0, _tab.getLoadingFunc)(tabContainer);
  var minWidth = args.minWidth;
  var minHeight = args.minHeight; // Inject HTML

  (0, _dom.addHTML)(tabContainer, $('.source-content', args.container).innerHTML);
  var albumName = $('.album-name');
  var selectedAlbumName, loadMoreElt, tabVisible, disconnectionScheduled;
  var self = {
    onEnter: onEnter,
    onLeave: function onLeave() {
      tabVisible = false;
      (0, _tab.stopObserving)(events);
    },
    setSelectView: setSelectView,
    renderAlbums: renderAlbums,
    renderPhotos: renderPhotos,
    albumData: {},
    photoData: {},
    photoDataByAlbumId: {},
    failToConnect: function failToConnect() {
      loading(false);
      flash('connection_error');
    },
    failToLoad: function failToLoad() {
      loading(false);
      flash('loading_error');
    }
  };
  var events;

  function onEnter() {
    tabVisible = true;
    events = [(0, _dom.on)($('.connect'), 'click', onConnect), (0, _dom.on)(tabContainer, 'click', '.back', function () {
      return setSelectView('albums');
    }), (0, _dom.on)(tabContainer, 'click', '.album-pic', function (e) {
      var target = e.currTarget;
      selectedAlbumName = target.getAttribute('data-title');
      self.selectedAlbumId = target.getAttribute('data-album-id');
      var photoData = self.photoDataByAlbumId[self.selectedAlbumId];

      if (photoData) {
        $('.photos .list').innerHTML = '';
        setSelectView('photos');
        renderPhotos(photoData);
      } else {
        loadPhotos(self.selectedAlbumId);
      }
    }), (0, _dom.on)(tabContainer, 'click', '[data-next-page-info]', function (e) {
      var target = e.currTarget;
      var info = target.getAttribute('data-next-page-info');
      target.removeAttribute('data-next-page-info');
      'albums' === target.getAttribute('data-more') ? loadMoreAlbums(info) : loadMorePhotos(info);
      loadMoreElt = target; // We don't want to mess with the scroll position if "load more" is the last item in the list

      target.style.visibility = 'hidden';
    }), (0, _dom.on)(tabContainer, 'click', '.photo', function (e) {
      var target = e.currTarget;
      target.getAttribute('data-too-small') ? flash('too_small', {
        values: {
          min: "".concat(minWidth, "x").concat(minHeight)
        }
      }) : args.onSelect(target.getAttribute('data-photo'));
    })];
  }

  function scheduleDisconnect() {
    if (disconnectionScheduled) return;
    disconnectionScheduled = true;
    setTimeout(function () {
      self.albumData = {};
      self.photoData = {};
      self.photoDataByAlbumId = {};
      setSelectView('intro');

      if (tabVisible) {
        flash('timeout', {
          type: 'warning'
        });
      }

      disconnectionScheduled = false;
    }, DEFAULT_TIMEOUT);
  }

  function setSelectView(view) {
    (0, _tools.each)($$('.view'), _dom.hide);
    (0, _dom.show)($(".view.".concat(view)));
  }

  function renderAlbums(albumData) {
    scheduleDisconnect();
    loading(false);
    var albums = $('.albums .list');
    var length = $$('.album-pic', albums).length;
    (0, _dom.addHTML)(albums, albumData.albums.slice(length).map(function (a) {
      return "<div class=\"album-pic\" data-title=\"".concat(a.title, "\" data-album-id=\"").concat(a.id, "\" data-count=\"").concat(a.count, "\" style=\"background-image: url('").concat(a.picture, "')\"></div>");
    }).join('') + (albumData.nextPageInfo ? "<div data-more=\"albums\" data-next-page-info=\"".concat(albumData.nextPageInfo, "\" class=\"more\">").concat(i18n('more'), "</div>") : ''));
    (0, _dom.remove)(loadMoreElt);
    var noData = !length && !$('.album-pic', albums);
    (noData ? _dom.addClass : _dom.removeClass)(albums, 'no-data');

    if (noData) {
      albums.innerHTML = "<div>".concat(i18n('no_data'), "</div>");
    }
  }

  function renderPhotos(photoData) {
    scheduleDisconnect();
    loading(false);

    if (self.selectedAlbumId) {
      albumName.innerHTML = selectedAlbumName;
    } else {
      (0, _dom.hide)($('.action'));
    }

    var photos = $('.photos .list');
    var length = $$('.photo', photos).length;
    (0, _dom.addHTML)(photos, photoData.photos.slice(length).map(function (img) {
      var tooSmall = img.width && img.height ? img.width >= minWidth && img.height >= minHeight : true;
      return "<div class=\"photo\" ".concat(tooSmall ? '' : "data-too-small=\"true\" ", "title=\"").concat(img.title || '', "\" data-photo=\"").concat(img.photo, "\" style=\"background-image: url('").concat(img.thumbnail, "')\"></div>");
    }).join('') + (photoData.nextPageInfo && !$('.photos .list [data-next-page-info]') ? "<div data-more=\"photos\" data-next-page-info=\"".concat(photoData.nextPageInfo, "\" class=\"more\">").concat(i18n('more'), "</div>") : ''));
    (0, _dom.remove)(loadMoreElt);
    var noData = !self.selectedAlbumId && !length && !$('.photo', photos);
    (noData ? _dom.addClass : _dom.removeClass)(photos, 'no-data');

    if (noData) {
      photos.innerHTML = "<div>".concat(i18n('no_data'), "</div>");
    }
  }

  return self;
}