"use strict";

exports.__esModule = true;
exports.default = Facebook;

var _tab = require("../tools/tab");

var _source = _interopRequireWildcard(require("../source"));

var _request = require("../tools/request");

var _image = require("../tools/image");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

var IMG_PROP = _image.webPSupport ? 'webp_images' : 'images';

function Facebook(tabContainer, args) {
  var options = args.options.facebook;
  var minSize = args.options.cropper.resizeToWidth || 1024;
  var $ = (0, _tab.getQueryFunc)(tabContainer);
  var loading = (0, _tab.getLoadingFunc)(tabContainer);
  var source = (0, _source.default)(tabContainer, args, loadSDK, loadMorePhotos, loadPhotos, loadMoreAlbums);

  function loadSDK(callback) {
    if ('undefined' !== typeof FB) return authenticate();
    loading();

    window.fbAsyncInit = function () {
      loading(false);
      FB.init({
        appId: options.appId,
        autoLogAppEvents: true,
        xfbml: true,
        version: 'v3.2'
      });
      authenticate();
    };

    (function (d, s) {
      var js = d.createElement(s);
      var fjs = d.getElementsByTagName(s)[0];
      js.src = "https://connect.facebook.net/".concat(options.locale, "/sdk.js");
      fjs.parentNode.insertBefore(js, fjs);
    })(document, 'script');
  }

  function authenticate() {
    FB.getLoginStatus(function (response) {
      loading();
      response.status !== 'connected' ? login() : loadAlbums();
    });
  }

  function login() {
    FB.login(function (response) {
      response.authResponse ? loadAlbums() : source.failToConnect();
    }, {
      scope: 'user_photos'
    });
  }

  function processAlbumData(response) {
    var result = response.data.map(function (album) {
      processPhotoData(album.photos || {
        data: [],
        paging: {}
      }, album.id);
      return {
        id: album.id,
        count: album.count,
        picture: album.cover_photo ? album.cover_photo.picture : 'about:blank',
        title: album.name
      };
    });
    source.albumData = {
      albums: (source.albumData.albums || []).concat(result),
      nextPageInfo: response.paging.next
    };
  }

  function processPhotoData(response, albumId) {
    var result = response.data.map(function (photo) {
      var pic = selectPhoto(photo[IMG_PROP], minSize);
      return {
        thumbnail: photo.picture,
        photo: pic.source,
        width: +pic.width,
        height: +pic.height
      };
    });
    source.photoDataByAlbumId[albumId] = {
      photos: ((source.photoDataByAlbumId[albumId] || {}).photos || []).concat(result),
      nextPageInfo: response.paging.next
    };
  }

  function selectPhoto(array, minSize) {
    var max;

    for (var i = 1, image, len = array.length; i < len; ++i) {
      image = array[i];
      if (image.width <= minSize || image.height <= minSize) return array[i - 1];
      if (!max || image.width > max.width && image.height > max.height) max = image;
    }

    return max;
  }

  function loadMore(url, success) {
    loading();
    (0, _request.sendRequest)(url, function (json) {
      loading(false);
      success(json);
    }, source.failToLoad);
  }

  function loadMoreAlbums(nextPageInfo) {
    loadMore(nextPageInfo, function (json) {
      processAlbumData(json, source.selectedAlbumId);
      source.renderAlbums(source.albumData);
    });
  }

  function loadAlbums() {
    loading();
    var limit = options.itemsToLoad || _source.DEFAULT_LOAD_ITEMS;
    FB.api("/me?fields=albums.limit(".concat(limit, "){name,count,cover_photo{picture},photos.limit(").concat(limit, "){picture,").concat(IMG_PROP, "}}"), function (response) {
      if (!response || response.error) {
        return source.failToLoad();
      }

      processAlbumData(response.albums || {
        data: [],
        paging: {}
      });
      source.setSelectView('albums');
      source.renderAlbums(source.albumData);
    });
  }

  function loadMorePhotos(nextPageInfo) {
    loadMore(nextPageInfo, function (json) {
      processPhotoData(json, source.selectedAlbumId);
      source.renderPhotos(source.photoDataByAlbumId[source.selectedAlbumId]);
    });
  }

  function loadPhotos(albumId) {
    $('.photos .list').innerHTML = '';
    source.setSelectView('photos');
    source.renderPhotos(source.photoDataByAlbumId[albumId]);
  }

  return source;
}