"use strict";

exports.__esModule = true;
exports.default = GooglePhotos;

var _tab = require("../tools/tab");

var _source = _interopRequireWildcard(require("../source"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function GooglePhotos(tabContainer, args) {
  var options = args.options.googlePhotos;
  var $ = (0, _tab.getQueryFunc)(tabContainer);
  var loading = (0, _tab.getLoadingFunc)(tabContainer);
  var source = (0, _source.default)(tabContainer, args, loadSDK, loadMorePhotos, loadPhotos, loadMoreAlbums);

  function loadSDK() {
    if ('undefined' !== typeof gapi) return authenticate();
    loading();

    window.gpAsyncInit = function () {
      authenticate();
    };

    (function (d, s) {
      var js = d.createElement(s);
      var fjs = d.getElementsByTagName(s)[0];
      js.src = 'https://apis.google.com/js/client:plus.js?onload=gpAsyncInit';
      fjs.parentNode.insertBefore(js, fjs);
    })(document, 'script');
  }

  function authenticate() {
    gapi.auth.authorize({
      immediate: false,
      client_id: options.oAuthClientID,
      scope: ['https://www.googleapis.com/auth/photoslibrary.readonly', 'profile']
    }, function (response) {
      if (response && !response.error) {
        loadAlbums();
      } else {
        source.failToConnect();
      }
    });
  }

  function processAlbumData(response) {
    var result = (response.albums || []).map(function (album) {
      return {
        id: album.id,
        count: album.mediaItemsCount,
        picture: "".concat(album.coverPhotoBaseUrl, "=w150-h150-c"),
        title: album.title
      };
    });
    source.albumData = {
      albums: (source.albumData.albums || []).concat(result),
      nextPageInfo: response.nextPageToken
    };
  }

  function processPhotoData(response, albumId) {
    var result = (response.mediaItems || []).reduce(function (acc, photo) {
      if (photo.mimeType.indexOf('video') < 0) {
        // TODO: +video?
        acc.push({
          title: photo.filename,
          thumbnail: "".concat(photo.baseUrl, "=w150-h150-c"),
          photo: photo.baseUrl,
          width: +photo.mediaMetadata.width,
          height: +photo.mediaMetadata.height
        });
      }

      return acc;
    }, []);
    source.photoDataByAlbumId[albumId] = {
      photos: ((source.photoDataByAlbumId[albumId] || {}).photos || []).concat(result),
      nextPageInfo: response.nextPageToken
    };
  }

  function loadMoreAlbums(nextPageInfo) {
    loadAlbums(nextPageInfo);
  }

  function loadAlbums(nextPageInfo) {
    loading();
    gapi.client.request({
      path: 'https://photoslibrary.googleapis.com/v1/albums',
      method: 'GET',
      params: {
        pageSize: options.itemsToLoad || _source.DEFAULT_LOAD_ITEMS,
        pageToken: nextPageInfo
      }
    }).then(function (response) {
      processAlbumData(response.result);
      source.setSelectView('albums');
      source.renderAlbums(source.albumData);
    }, source.failToLoad);
  }

  function loadMorePhotos(nextPageInfo) {
    loadPhotos(source.selectedAlbumId, nextPageInfo);
  }

  function loadPhotos(albumId, nextPageInfo) {
    loading();
    gapi.client.request({
      path: 'https://photoslibrary.googleapis.com/v1/mediaItems:search',
      method: 'POST',
      params: {
        pageSize: options.itemsToLoad || _source.DEFAULT_LOAD_ITEMS,
        pageToken: nextPageInfo,
        albumId: albumId
      }
    }).then(function (response) {
      processPhotoData(response.result, albumId);
      $('.photos .list').innerHTML = '';
      source.setSelectView('photos');
      source.renderPhotos(source.photoDataByAlbumId[albumId]);
    }, source.failToLoad);
  }

  return source;
}