"use strict";

exports.__esModule = true;
exports.default = InstagramPictureManager;

var _tab = require("../tools/tab");

var _source = _interopRequireWildcard(require("../source"));

var _popup = require("../tools/popup");

var _request = require("../tools/request");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function InstagramPictureManager(tabContainer, args) {
  var options = args.options.instagram;
  var loading = (0, _tab.getLoadingFunc)(tabContainer);
  var source = (0, _source.default)(tabContainer, args, authenticate, loadMorePhotos);
  var accessToken, userID; // We'll open instagram sign-in page in a new popup
  // Once authenticated, instagram will redirect us to redirectURI and will add an access_token in the URL
  // We'll detect the popup location change and extract the access_token
  // Note:
  //  - We can only detect the popup location change if we keep the same origin.
  //  - The content at this URL (redirectURI) doesn't matter, we'll close the popup as soon as we get the accessToken
  //  - Ensure that you correctly setup your instagram app (Security -> Valid redirect URI + Enable Implicit OAuth)

  function authenticate() {
    var pop = (0, _popup.popup)("https://api.instagram.com/oauth/authorize/?client_id=".concat(options.clientID, "&redirect_uri=").concat(encodeURIComponent(options.redirectURI || "".concat(window.location.protocol, "//").concat(window.location.host)), "&response_type=token"), 'Instagram', 600, 497);
    var interval = accessToken ? 20 : 250;

    var waitAccessToken = function waitAccessToken() {
      setTimeout(function () {
        try {
          if (!pop.location || !pop.location.href) return; // window has been closed

          if (pop.location.href.search(/[\?&]error=/) >= 0) {
            pop.close();
            return source.failtToConnect();
          }

          if (!pop.location.hash || pop.location.hash.indexOf('#access_token') < 0) return waitAccessToken();
          accessToken = pop.location.hash.replace('#access_token=', '');
          userID = accessToken.slice(0, accessToken.indexOf('.'));
          pop.close();
          loadPhotos();
        } catch (e) {
          waitAccessToken();
        }
      }, interval);
    };

    setTimeout(function () {
      return interval = 500;
    }, 3 * 60 * 1000);
    setTimeout(function () {
      return interval = 1000;
    }, 10 * 60 * 1000);
    waitAccessToken();
  }

  function processPhotoData(response, albumId) {
    var result = response.data.map(function (post) {
      var pic = post.images.standard_resolution;
      return {
        title: post.caption.text,
        thumbnail: post.images.thumbnail.url,
        photo: pic.url,
        width: +pic.width,
        height: +pic.height
      };
    });
    source.photoData = {
      photos: (source.photoData.photos || []).concat(result),
      nextPageInfo: response.pagination.next_url
    };
  }

  function loadMorePhotos(nextPageInfo) {
    loadPhotos(nextPageInfo);
  }

  function loadPhotos(nextPageInfo) {
    loading();
    (0, _request.sendRequest)(nextPageInfo || "https://api.instagram.com/v1/users/".concat(userID, "/media/recent?access_token=").concat(accessToken, "&count=").concat(options.itemsToLoad || _source.DEFAULT_LOAD_ITEMS), function (response) {
      processPhotoData(response);
      source.setSelectView('photos');
      source.renderPhotos(source.photoData);
    }, source.failToLoad);
  }

  return source;
}