"use strict";

exports.__esModule = true;
exports.hide = hide;
exports.show = show;
exports.addClass = addClass;
exports.removeClass = removeClass;
exports.remove = remove;
exports.on = on;
exports.addHTML = void 0;

var _isNode = _interopRequireDefault(require("./is-node"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _addHTML = function _addHTML(elt, html) {
  elt.innerHTML += html;
  return elt;
};

if (!_isNode.default) {
  var proto = Element.prototype;

  if (!proto.matches) {
    proto.matches = proto.matchesSelector || proto.mozMatchesSelector || proto.msMatchesSelector || proto.oMatchesSelector || proto.webkitMatchesSelector;
  }

  if (proto.insertAdjacentHTML) {
    _addHTML = function _addHTML(elt, html) {
      elt.insertAdjacentHTML('beforeend', html);
      return elt;
    };
  }
}

var addHTML = _addHTML;
exports.addHTML = addHTML;

function hide(elt) {
  elt.style.display = 'none';
  return elt;
}

function show(elt) {
  elt.style.display = '';
  return elt;
}

function addClass(elt, className) {
  if (" ".concat(elt.className, " ").indexOf(" ".concat(className, " ")) < 0) elt.className += " ".concat(className);
  return elt;
}

function removeClass(elt, className) {
  className = " ".concat(className, " ");
  var cName = " ".concat(elt.className, " ");

  while (cName.indexOf(className) >= 0) {
    cName = cName.replace(className, ' ');
  }

  elt.className = cName.trim();
  return elt;
}

function remove(elt) {
  elt && elt.parentNode && elt.parentNode.removeChild(elt);
}

function on(elt, eventNames, selector, handler, options) {
  var func = selector;

  if ('function' !== typeof selector) {
    var selectorPlus = selector.split(',').map(function (s) {
      return s + ' *';
    }).join(',');

    func = function func(event) {
      if (event.target.matches(selector)) {
        event.currTarget = event.target;
        handler(event);
      } else if (event.target.matches(selectorPlus)) {
        for (var elem = event.target; elem && elem !== elt; elem = elem.parentNode) {
          if (elem.matches(selector)) event.currTarget = elem;
        }

        handler(event);
      }
    };
  } else {
    options = handler;
  }

  eventNames = eventNames.split(',');
  eventNames.forEach(function (eventName) {
    return elt.addEventListener(eventName, func, options || false);
  });
  return function () {
    return eventNames.forEach(function (eventName) {
      elt.removeEventListener(eventName, func);
    });
  };
}